package Product

import (
	"AIOTCOER/logs"
	"AIOTCOER/models"
	"github.com/astaxie/beego/cache"
	_ "github.com/astaxie/beego/cache/redis"
	"github.com/beego/beego/v2/adapter/orm"
	orm2 "github.com/beego/beego/v2/client/orm"
	_ "github.com/go-sql-driver/mysql"
)

// OTA
type ProductOTA struct {
	Id          int    `orm:"column(ID);size(11);auto;pk" json:"Id" form:"Id"`
	T_ProductID string `orm:"size(8);index" json:"T_ProductID" form:"T_ProductID"`        // 产品型号 随机生成（8位）
	T_name      string `orm:"size(256);" json:"T_name" form:"T_name"`                     // 固件名称
	T_version   string `orm:"size(200);" json:"T_version" form:"T_version"`               // 版本
	T_file      string `orm:"type(text);default('')" json:"T_file" form:"T_file"`         // 文件
	T_hash      string `orm:"type(text);default('')" json:"T_hash" form:"T_hash"`         // 文件 MD5
	T_size      int    `orm:"size(256);" json:"T_size" form:"T_size"`                     // 文件大小  字节
	T_describe  string `orm:"type(text);default('')" json:"T_describe" form:"T_describe"` // 描述
	T_publish   bool   `orm:"" json:"T_publish" form:"T_publish"`                         // 全量发布（类型所有设备 全部更新）默认：不发布 \ 全量发布

	CreateTime models.Time `orm:"column(create_time);type(timestamp);auto_now_add" json:"CreateTime"`
}

func (t *ProductOTA) TableName() string {
	return "ProductOTA" // 数据库名称   // ************** 替换 FormulaList **************
}

var redis_ProductOTA cache.Cache

func init() {
	//注册模型
	orm.RegisterModel(new(ProductOTA))

}

// 添加
func (t *ProductOTA) Add() (is bool) {
	o := orm.NewOrm()

	id, err := o.Insert(t)
	if err != nil {
		return false
	}

	println(id)
	return true
}

// 获取
func (t *ProductOTA) Read() (bool bool) {
	o := orm.NewOrm()
	err := o.Read(t) // o.Read(&r,"Tokey") 如果不是 主键 就得指定字段名
	if err != nil {
		return false
	}

	return true
}

// 删除
func (t *ProductOTA) Delete() bool {
	o := orm.NewOrm()
	if num, err := o.Delete(t); err == nil {
		logs.Println("Number of records deleted in database:", num)
	} else {
		return false
	}

	return true
}

// 获取列表
func (t *ProductOTA) Lists(PageIndex int, PageSize int) (r []ProductOTA, Total int64) {

	o := orm.NewOrm()

	// 也可以直接使用 Model 结构体作为表名
	qs := o.QueryTable(new(ProductOTA))
	var offset int64
	if PageIndex <= 1 {
		offset = 0
	} else {
		offset = int64((PageIndex - 1) * PageSize)
	}

	// 筛选参数
	cond := orm.NewCondition()
	if len(t.T_name) > 0 {
		cond = cond.And("T_name__icontains", t.T_name) // .AndNot("status__in", 1).Or("profile__age__gt", 2000)
	}
	if len(t.T_version) > 0 {
		cond = cond.And("T_version__icontains", t.T_version) // .AndNot("status__in", 1).Or("profile__age__gt", 2000)
	}

	// 执行
	qs.Limit(PageSize, offset).SetCond((*orm2.Condition)(cond)).OrderBy("-CreateTime").All(&r)
	Total, _ = qs.SetCond((*orm2.Condition)(cond)).Count()

	return r, Total
}
