package Product

import (
	"AIOTCOER/conf"
	"AIOTCOER/logs"
	"encoding/json"
	"fmt"
	"github.com/astaxie/beego/cache"
	_ "github.com/astaxie/beego/cache/redis"
	"github.com/beego/beego/v2/adapter/orm"
	orm2 "github.com/beego/beego/v2/client/orm"
	_ "github.com/go-sql-driver/mysql"
	"time"
)

// 产品协议
type ProductProt struct {
	Id         int    `orm:"column(ID);size(11);auto;pk" json:"Id" form:"Id"`
	T_uuid     string `orm:"size(8);" json:"T_uuid" form:"T_uuid"`                  // 用户识别码（8位） ，管理员可以为空

	T_lang     int    `orm:"size(1);default(1)" json:"T_lang" form:"T_lang"`             //编程语言 0: 无解析 1: go   2: C
	T_analysis string `orm:"size(100);default('')" json:"T_analysis" form:"T_analysis"`  // 插件 SO 文件名
	T_text     string `orm:"type(text);default('')" json:"T_text" form:"T_text"`         // 代码内容
	T_describe string `orm:"type(text);default('')" json:"T_describe" form:"T_describe"` // 描述内容
}

func (t *ProductProt) TableName() string {
	return "ProductProt" // 数据库名称   // ************** 替换 FormulaList **************
}

var redis_ProductProt cache.Cache

func init() {
	//注册模型
	orm.RegisterModel(new(ProductProt))

	config := fmt.Sprintf(`{"key":"%s","conn":"%s","dbNum":"%s","password":"%s"}`,
		"ProductProt", conf.Redis_address, conf.Redis_dbNum, conf.Redis_password)
	var err error
	redis_ProductProt, err = cache.NewCache("redis", config)
	if err != nil || redis_ProductProt == nil {
		logs.PrintlnError(config, err)
		panic(any(err))
	}
}

// ---------------- Redis -------------------
func (t *ProductProt) redis_Set() (err error) {
	//json序列化
	str, err := json.Marshal(t)
	if err != nil {
		logs.PrintlnError("Redis_Set", err)
		return
	}

	err = redis_ProductProt.Put(fmt.Sprintf("%d", t.Id), str, 24*time.Hour)
	if err != nil {
		logs.Println("set key:", t.Id, ",value:", str, err)
	}
	return
}
func (t *ProductProt) redis_Get(key string) (is bool) {
	if redis_ProductProt.IsExist(key) {
		//println("找到key:",key)
		v := redis_ProductProt.Get(key)
		if v == nil {
			return false
		}
		json.Unmarshal(v.([]byte), t)
		return true
	}
	//println("没有 找到key:",key)
	return false
}
func (t *ProductProt) redis_DelK() (err error) {
	err = redis_ProductProt.Delete(fmt.Sprintf("%d", t.Id))
	return
}

// ---------------- 方法 -------------------

// 添加
func (t *ProductProt) Add() ( int, bool) {
	o := orm.NewOrm()

	id, err := o.Insert(t)
	if err != nil {
		return 0,false
	}
	println(id)
	t.redis_Set()
	return int(id),true
}

// 获取
func (t *ProductProt) Read() (bool bool) {
	if t.redis_Get(fmt.Sprintf("%d", t.Id)) {
		return true
	}

	if !(t.Id > 0) {
		logs.Println("ProductProt.Id:", t.Id)
		return false
	}
	o := orm.NewOrm()
	err := o.Read(t) // o.Read(&r,"Tokey") 如果不是 主键 就得指定字段名
	if err != nil {
		return false
	}
	t.redis_Set() // Redis 更新缓存
	return true
}

// 修改
func (t *ProductProt) Update(cols ...string) bool {
	o := orm.NewOrm()
	if num, err := o.Update(t, cols...); err == nil {
		logs.Println("Number of records updated in database:", num)
		t.redis_Set() // Redis 更新缓存
		return true
	}
	return false
}

// 删除
func (t *ProductProt) Delete() bool {
	o := orm.NewOrm()
	if num, err := o.Delete(t); err == nil {
		logs.Println("Number of records deleted in database:", num)
	} else {
		return false
	}

	t.redis_DelK()
	return true
}

// 获取列表
func (t *ProductProt) Lists(PageIndex int, PageSize int) (r []ProductProt, Total int64) {

	o := orm.NewOrm()

	// 也可以直接使用 Model 结构体作为表名
	qs := o.QueryTable(new(ProductProt))
	var offset int64
	if PageIndex <= 1 {
		offset = 0
	} else {
		offset = int64((PageIndex - 1) * PageSize)
	}

	// 筛选参数
	cond := orm.NewCondition()
	if len(t.T_uuid) == 8 {
		cond = cond.And("T_uuid", t.T_uuid) // 用户识别码
	}

	// 执行
	qs.Limit(PageSize, offset).SetCond((*orm2.Condition)(cond)).All(&r)
	Total, _ = qs.SetCond((*orm2.Condition)(cond)).Count()

	return r, Total
}

// 获取列表
func (t *ProductProt) Lists_All() (r []ProductProt) {

	o := orm.NewOrm()

	// 也可以直接使用 Model 结构体作为表名
	qs := o.QueryTable(new(ProductProt))

	// 筛选参数
	cond := orm.NewCondition()

	if len(t.T_uuid) == 8 {
		cond = cond.And("T_uuid", t.T_uuid) // 用户识别码
	}
	// 执行
	qs.SetCond((*orm2.Condition)(cond)).All(&r)

	return r
}
