package Product

import (
	"AIOTCOER/conf"
	"AIOTCOER/lib"
	"AIOTCOER/logs"
	"AIOTCOER/models"
	"encoding/json"
	"fmt"
	"github.com/astaxie/beego/cache"
	_ "github.com/astaxie/beego/cache/redis"
	"github.com/beego/beego/v2/adapter/orm"
	orm2 "github.com/beego/beego/v2/client/orm"
	_ "github.com/go-sql-driver/mysql"
	"github.com/vmihailenco/msgpack/v5"
	"time"
)

// 产品类型
type ProductType struct {
	T_ProductID  string `orm:"size(8);pk" json:"T_ProductID" msgpack:"T_ProductID" form:"T_ProductID"` // 产品型号 随机生成（8位）
	T_ProductKey string `orm:"size(56);" json:"T_ProductKey" msgpack:"T_ProductKey" `                  // 授权密钥
	T_uuid       string `orm:"size(64);index" json:"-" msgpack:"T_uuid" form:"-"`                      // 用户识别码（ < 64位） ，管理员可以为空

	T_name string `orm:"size(256);" json:"T_name" msgpack:"T_name" form:"T_name"` // 产品名称
	//T_img   string `orm:"size(256);" json:"T_img" form:"T_img"`            // 产品图片
	T_mode  int    `orm:"size(3);default(0)" json:"T_mode" msgpack:"T_mode" form:"T_mode"`  //接入网关ID
	T_prot  int    `orm:"size(11);default(0)" json:"T_prot" msgpack:"T_prot" form:"T_prot"` //接入协议ID  0：统一协议  ProductProt ID
	T_panel string `orm:"size(32);" json:"T_panel" msgpack:"T_panel" form:"T_panel"`        // 产品面板ID

	// 功能模型
	T_TabData     string                   `orm:"type(text);default('[]')" json:"T_TabData" msgpack:"T_TabData" form:"T_TabData"` // 产品模型
	T_TabDataJson []map[string]interface{} `orm:"-" json:"T_TabDataJson" msgpack:"T_TabDataJson"`                                 // 产品模型 json
	T_save        map[string]int           `orm:"-" json:"T_save" msgpack:"T_save"`                                               // 保存天数  数据保存天数，默认为30天， 0为永久保存
	T_filter      []string                 `orm:"-" json:"T_filter" msgpack:"T_filter"`                                           // 数据过滤 上传值与最新的值一样将会被忽略， 不会保存到数据库
	T_renew       []string                 `orm:"-" json:"T_renew" msgpack:"T_renew"`                                             // 平台托管   平台->设备 时将参数强制更新此参数,用于 参数托管给平台记录

	T_state    int         `orm:"size(1);default(1);index" json:"-"  msgpack:"-" form:"-"` //  1 正常  0 禁用\删除  (管理员)
	CreateTime models.Time `orm:"column(create_time);type(timestamp);auto_now_add" json:"CreateTime" msgpack:"CreateTime"`
	UpdateTime models.Time `orm:"column(update_time);type(timestamp);auto_now" json:"UpdateTime" msgpack:"UpdateTime"`
}

/*  产品模型  T_TabData \ T_TabDataJson
[
	{
		"T_name": "温度",
		"T_tab": "Tt",
		"T_type": 1,
		"T_attr": {
					"unit": "C",
					"dvalue": 0,
					"accuracy": 1,
					"min": 2,
					"max": 8,
					},
		"T_text": "温度温度温度",
		"T_read": true,
		"children":[{
			"T_name": "采样率",
			"T_tab": "Tt_speed",
			"T_type": 1,
			"T_attr": {
						"unit": "s",
						"dvalue": 15,
						"accuracy": 1,
						"min": 0,
						"max": 30,
						},
			"T_text": "数据采集速度",
			"T_read": false,
			"children":[]
		}]
	}

]

// T_attr属性  定义

	1 数值
		unit 单位
		dvalue 默认值
		accuracy 精度
		min 最小值
		max 最大值

	2 文本
		dvalue 默认值

	3 枚举
		map [] 枚举
			value 值
			text 描述
*/

// 产品类型 - 列表
type ProductType_List struct {
	T_ProductID string `orm:"size(8);pk" json:"T_ProductID" form:"T_ProductID"` // 产品型号 随机生成（8位）

	T_name string `orm:"size(256);" json:"T_name" form:"T_name"` // 产品名称
	//T_img  string `orm:"size(256);" json:"T_img" form:"T_img"`            // 产品图片
	T_prot int `orm:"size(10);default(0)" json:"T_prot" form:"T_prot"` //接入协议ID  0：统一协议
	T_mode int `orm:"size(10);default(0)" json:"T_mode" form:"T_mode"` //接入网关ID

	CreateTime models.Time `orm:"column(create_time);type(timestamp);auto_now_add" json:"CreateTime"`
	UpdateTime models.Time `orm:"column(update_time);type(timestamp);auto_now" json:"UpdateTime"`
}

// 产品类型
type ProductType_R struct {
	T_name string `orm:"size(256);" json:"T_name" msgpack:"T_name" form:"T_name"` // 产品名称

	T_panel string `orm:"size(32);" json:"T_panel" msgpack:"T_panel" form:"T_panel"` // 产品面板ID

	// 功能模型
	T_TabDataJson []map[string]interface{} `orm:"-" json:"T_TabDataJson" msgpack:"T_TabDataJson"`                                 // 产品模型 json

	CreateTime models.Time `orm:"column(create_time);type(timestamp);auto_now_add" json:"CreateTime" msgpack:"CreateTime"`
	UpdateTime models.Time `orm:"column(update_time);type(timestamp);auto_now" json:"UpdateTime" msgpack:"UpdateTime"`
}

func (t *ProductType) TableName() string {
	return "ProductType" // 数据库名称   // ************** 替换 FormulaList **************
}

var redis_ProductType cache.Cache

func init() {
	//注册模型
	orm.RegisterModel(new(ProductType))

	config := fmt.Sprintf(`{"key":"%s","conn":"%s","dbNum":"%s","password":"%s"}`,
		"redis_ProductType", conf.Redis_address, conf.Redis_dbNum, conf.Redis_password)
	var err error
	redis_ProductType, err = cache.NewCache("redis", config)
	if err != nil || redis_ProductType == nil {
		logs.PrintlnError(config, err)
		panic(any(err))
	}
}

// ---------------- Redis -------------------
func (t *ProductType) redis_Set() (err error) {

	if len(t.T_TabData) > 0 {
		var T_datajson []map[string]interface{}
		json.Unmarshal([]byte(t.T_TabData), &T_datajson)
		t.T_TabDataJson = T_datajson
	}

	// 汇总 数据过滤
	t.T_filter = []string{}
	lib.Hande_Tab_Tfilter(t.T_TabDataJson, "", &t.T_filter)

	// 汇总 平台托管
	//t.T_renew = make(map[string]bool)
	t.T_renew = []string{}
	lib.Hande_Tab_Trenew(t.T_TabDataJson, "", &t.T_renew)
	logs.Println("T_renew:", t.T_renew)

	// 汇总 保存天数
	t.T_save = make(map[string]int)
	lib.Hande_Tab_Tsave(t.T_TabDataJson, "", &t.T_save)

	//json序列化
	str, err := msgpack.Marshal(t)
	if err != nil {
		logs.PrintlnError("Redis_Set", err)
		return
	}

	err = redis_ProductType.Put(t.T_ProductID, str, 24*time.Hour)
	if err != nil {
		logs.Println("set key:", t.T_ProductID, ",value:", str, err)
	}
	return
}
func (t *ProductType) redis_Get(key string) (is bool) {
	if redis_ProductType.IsExist(key) {
		//println("找到key:",key)
		v := redis_ProductType.Get(key)
		if v == nil {
			return false
		}
		msgpack.Unmarshal(v.([]byte), t)
		return true
	}
	//println("没有 找到key:",key)
	return false
}
func (t *ProductType) Redis_DelK() (err error) {
	err = redis_ProductType.Delete(t.T_ProductID)
	return
}

// ---------------- 方法 -------------------

// 添加
func (t *ProductType) Add() (is bool) {
	o := orm.NewOrm()

	id, err := o.Insert(t)
	if err != nil {
		return false
	}
	println(id)
	t.redis_Set()
	return true
}

// 获取
func (t *ProductType) Read() (bool bool) {
	if t.redis_Get(t.T_ProductID) {
		return true
	}
	o := orm.NewOrm()
	err := o.Read(t) // o.Read(&r,"Tokey") 如果不是 主键 就得指定字段名
	if err != nil {
		return false
	}
	t.redis_Set() // Redis 更新缓存
	return true
}

// 修改
func (t *ProductType) Update(cols ...string) bool {
	o := orm.NewOrm()
	num, err := o.Update(t, cols...)
	if err == nil {
		logs.Println("Number of records updated in database:", num)
		t.Redis_DelK() // Redis 更新缓存

		return true
	}
	logs.PrintlnError("ProductType Update", err)
	return false
}

// 获取列表
func (t *ProductType) Lists(PageIndex int, PageSize int) (r_ []ProductType_List, Total int64) {

	o := orm.NewOrm()

	// 也可以直接使用 Model 结构体作为表名
	qs := o.QueryTable(new(ProductType))
	var offset int64
	if PageIndex <= 1 {
		offset = 0
	} else {
		offset = int64((PageIndex - 1) * PageSize)
	}

	// 筛选参数
	cond := orm.NewCondition().And("T_state", 1) //  1 正常  2 禁用\删除
	if len(t.T_name) > 0 {
		cond = cond.And("T_name__icontains", t.T_name) // .AndNot("status__in", 1).Or("profile__age__gt", 2000)
	}
	if len(t.T_uuid) >= 8 {
		cond = cond.And("T_uuid", t.T_uuid) // 用户识别码
	}
	var r []ProductType
	// 执行
	qs.Limit(PageSize, offset).SetCond((*orm2.Condition)(cond)).OrderBy("-CreateTime").All(&r)
	Total, _ = qs.SetCond((*orm2.Condition)(cond)).Count()

	for _, v := range r {
		tr := ProductType_List{
			T_ProductID: v.T_ProductID,
			T_name:      v.T_name,
			T_prot:      v.T_prot,
			T_mode:      v.T_mode,
			CreateTime:  v.CreateTime,
			UpdateTime:  v.UpdateTime,
		}
		r_ = append(r_, tr)

	}
	return r_, Total
}

// 获取列表
func (t *ProductType) Lists_All() (r []ProductType) {

	o := orm.NewOrm()

	// 也可以直接使用 Model 结构体作为表名
	qs := o.QueryTable(new(ProductType))

	// 筛选参数
	cond := orm.NewCondition()
	if len(t.T_name) > 0 {
		cond = cond.And("T_name__icontains", t.T_name) // .AndNot("status__in", 1).Or("profile__age__gt", 2000)
	}
	if len(t.T_ProductID) > 0 {
		cond = cond.And("T_ProductID", t.T_ProductID) // .AndNot("status__in", 1).Or("profile__age__gt", 2000)
	}
	if len(t.T_uuid) >= 8 {
		cond = cond.And("T_uuid", t.T_uuid) // 用户识别码
	}
	// 执行
	qs.SetCond((*orm2.Condition)(cond)).OrderBy("-CreateTime").All(&r)
	for i, _ := range r {
		if len(r[i].T_TabData) > 0 {
			var T_datajson []map[string]interface{}
			json.Unmarshal([]byte(r[i].T_TabData), &T_datajson)
			r[i].T_TabDataJson = T_datajson
		}

	}
	return r
}
