package StatisticalData

import (
	"AIOTCOER/models"
	_ "github.com/astaxie/beego/cache/redis"
	"github.com/beego/beego/v2/adapter/orm"
	orm2 "github.com/beego/beego/v2/client/orm"
	_ "github.com/go-sql-driver/mysql"
	"strings"
)

// 流量统计（每天）
type FlowCount struct {
	Id          int    `orm:"column(ID);size(11);auto;pk" json:"Id" form:"Id"`
	T_ProductID string `orm:"size(8);" json:"T_ProductID" form:"T_ProductID"` // 产品型号 随机生成（8位）

	T_send    int64 `orm:"default(0)" json:"T_datat"`  //设备发送 字节数
	T_receive int64 `orm:"default(0)" json:"T_datar"`  //设备接受 字节数
	T_relay   int64 `orm:"default(0)" json:"T_relays"` //转发次数

	CreateTime models.Time `orm:"column(create_time);type(timestamp);auto_now_add" json:"CreateTime"`
}
type FlowCount_T struct {
	T_send    int64
	T_receive int64
	T_relay   int64
}

var FlowCountMap map[string]*FlowCount_T

func (t *FlowCount) TableName() string {
	return "FlowCount" // 数据库名称   // ************** 替换 FormulaList **************
}

func init() {
	//注册模型
	orm.RegisterModel(new(FlowCount))
	FlowCountMap = make(map[string]*FlowCount_T)

}

// 每天定时统计
func FlowCountHandle() {
	for k, v := range FlowCountMap {
		// 只统计  有流量的
		//if v.T_send == 0 && v.T_receive == 0 && v.T_relay == 0 {
		//	continue
		//}
		r := FlowCount{
			T_ProductID: k,
			T_send:      v.T_send,
			T_receive:   v.T_receive,
			T_relay:     v.T_relay,
		}
		r.Add()
		// 删除缓存
		delete(FlowCountMap, k)
	}

}

// 添加
func (t *FlowCount) Add() (is bool) {
	o := orm.NewOrm()
	_, err := o.Insert(t)
	if err != nil {
		return false
	}
	return true
}

// 获取列表
func FlowCountLists(T_ProductID string, PageSize int) (r []FlowCount) {

	o := orm.NewOrm()

	// 也可以直接使用 Model 结构体作为表名
	qs := o.QueryTable(new(FlowCount))

	// 筛选参数
	cond := orm.NewCondition()

	// 多产品查询
	if len(T_ProductID) == 0 {
		return []FlowCount{}
	}
	for _, v := range strings.Split(T_ProductID, ",") {
		if len(v) > 0 {
			cond = cond.And("T_ProductID", v) // .AndNot("status__in", 1).Or("profile__age__gt", 2000)
		}
	}

	// == 1 代表只获取今天
	if PageSize > 1 {
		// 执行
		qs.Limit(PageSize-1, 0).SetCond((*orm2.Condition)(cond)).All(&r)

	}

	for _, v := range strings.Split(T_ProductID, ",") {
		if len(v) > 0 {
			v_r, is := FlowCountMap[v]
			if !is {
				r = append(r, FlowCount{
					T_ProductID: v,
					T_send:      0,
					T_receive:   0,
					T_relay:     0,
				})
				continue
			}
			r = append(r, FlowCount{
				T_ProductID: v,
				T_send:      v_r.T_send,
				T_receive:   v_r.T_receive,
				T_relay:     v_r.T_relay,
			})
		}
	}

	return r
}
